@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2015 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -1444,13 +1444,29 @@ Apollonius.getTangentsThroughPoint = function(circle, p) {
         var circle2 = RCircle.createFrom2Points(p, circle.center);
         var touchingPoints = circle2.getIntersectionPoints(circle, false);
         var lines = [];
-        lines[0] = new RLine(p, touchingPoints[1]);
-        lines[1] = new RLine(p, touchingPoints[0]);
+        if (!isNull(touchingPoints[1])) {
+            lines[0] = new RLine(p, touchingPoints[1]);
+        }
+        else {
+            lines[0] = undefined;
+        }
+
+        if (!isNull(touchingPoints[0])) {
+            lines[1] = new RLine(p, touchingPoints[0]);
+        }
+        else {
+            lines[1] = undefined;
+        }
+
         return lines;
     }
 };
 
 Apollonius.getParallelLinesWithDistance = function(line, distance) {
+    if (isNull(line)) {
+        return [];
+    }
+
     if (!isLineBasedShape(line)) {
         return [];
     }
