@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2015 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -273,44 +273,27 @@ DefaultAction.prototype.mouseMoveEvent = function(event) {
     }
 };
 
-DefaultAction.prototype.getEntityIdUnderCursor = function(event, range) {
-    var view = event.getGraphicsView();
-    if (isNull(range)) {
-        range = this.pickRangePixels;
-    }
-
-    range = view.mapDistanceFromView(range);
-
-    var strictRange = view.mapDistanceFromView(10);
-    return this.di.getClosestEntity(event.getModelPosition(), range, strictRange, false);
-};
-
 DefaultAction.prototype.mouseReleaseEvent = function(event) {
     var persistentSelection = RSettings.getBoolValue("GraphicsView/PersistentSelection", false);
     var view, range, strictRange, entityId;
 
-    var add = false;
-    if ((event.modifiers().valueOf() === Qt.ShiftModifier.valueOf()) ||
-            (event.modifiers().valueOf() === Qt.ControlModifier.valueOf()) ||
-            persistentSelection===true) {
-
-        add = true;
-    }
+    var shiftPressed = (event.modifiers() & Qt.ShiftModifier) > 0 || (event.modifiers() & Qt.ControlModifier) > 0 || persistentSelection===true;
 
     if (event.button() === Qt.LeftButton) {
         switch (this.state) {
         case DefaultAction.State.Dragging:
-            entityId = this.getEntityIdUnderCursor(event);
-            //qDebug("entity id: ", entityId);
+            entityId = this.getEntityId(event);
+            //qDebug("entityId: ", entityId);
+
             if (entityId !== -1) {
-                if (add && this.document.isSelected(entityId)) {
+                if (shiftPressed && this.document.isSelected(entityId)) {
                     this.deselectEntity(entityId);
                 }
                 else {
-                    this.selectEntity(entityId, add);
+                    this.selectEntity(entityId, shiftPressed);
                 }
             } else {
-                if (!add) {
+                if (!shiftPressed) {
                     if (persistentSelection===false) {
                         this.di.clearSelection();
                     }
@@ -358,24 +341,25 @@ DefaultAction.prototype.mouseReleaseEvent = function(event) {
         }
 
         else if (this.state===DefaultAction.State.Neutral) {
+            var appWin = EAction.getMainWindow();
             var rightClickRange = RSettings.getIntValue("GraphicsView/RightClickRange", 10);
-            entityId = this.getEntityIdUnderCursor(event, rightClickRange);
+            entityId = EAction.getEntityIdUnderCursor(this.di, event, rightClickRange);
 
             var rightClickToDeselect = RSettings.getBoolValue("GraphicsView/RightClickToDeselect", false);
-            var rightClickEntityContextMenu = RSettings.getBoolValue("GraphicsView/RightClickEntityContextMenu", false);
 
             // right-click on entity can be used to show context menu:
             if (entityId!==-1) {
                 // show entity context menu:
+                var rightClickEntityContextMenu = RSettings.getBoolValue("ContextMenu/RightClickEntityContextMenu", true);
                 if (rightClickEntityContextMenu) {
-                    var appWin = EAction.getMainWindow();
-                    appWin.showContextMenu(entityId);
+                    event.accept();
+                    appWin.showContextMenu(entityId, event.getModelPosition());
                     handled = true;
                 }
 
                 // use right-click on entity to select entity:
-                if (rightClickToDeselect) {
-                    this.selectEntity(entityId, add);
+                else if (rightClickToDeselect) {
+                    this.selectEntity(entityId, shiftPressed);
                     handled = true;
                 }
             }
@@ -393,7 +377,18 @@ DefaultAction.prototype.mouseReleaseEvent = function(event) {
         }
 
         if (!handled) {
-            CadToolBar.back();
+            if (CadToolBar.back()===true) {
+                handled = true;
+            }
+        }
+
+        if (!handled) {
+            // show context menu for empty space:
+            var rightClickContextMenu = RSettings.getBoolValue("ContextMenu/RightClickContextMenu", true);
+            if (rightClickContextMenu===true) {
+                event.accept();
+                appWin.showContextMenu(RObject.INVALID_ID, event.getModelPosition());
+            }
         }
     }
 };
@@ -435,11 +430,35 @@ DefaultAction.prototype.escapeEvent = function(event) {
 DefaultAction.prototype.pickCoordinate = function(event, preview) {
     var op;
 
+    var shiftPressed = (event.getModifiers() & Qt.ShiftModifier) > 0;
+    var doc = this.getDocument();
+
     switch (this.state) {
     case DefaultAction.State.MovingReference:
     case DefaultAction.State.SettingReference:
+
+        // find reference shape for lengthen, shorten when shift is pressed:
+        var referenceShape = undefined;
+        var d2 = event.getModelPosition();
+        if (shiftPressed) {
+            var view = event.getGraphicsView();
+            var modelRange = view.mapDistanceFromView(this.minPickRangePixels);
+            var referenceEntityId = this.di.getClosestEntity(this.d1Model, modelRange, 10, false, true);
+            if (referenceEntityId!==RObject.INVALID_ID) {
+                var e = doc.queryEntity(referenceEntityId);
+                if (!isNull(e)) {
+                    referenceShape = e.getClosestSimpleShape(this.d1Model);
+                }
+            }
+
+            if (!isNull(referenceShape)) {
+                d2 = referenceShape.getClosestPointOnShape(d2, false);
+                this.di.addAuxShapeToPreview(new RLine(this.d2Model, event.getModelPosition()));
+            }
+        }
+
         if (preview) {
-            this.d2Model = event.getModelPosition();
+            this.d2Model = d2;
             op = new RMoveReferencePointOperation(this.d1Model, this.d2Model);
             this.di.previewOperation(op);
         }
@@ -448,7 +467,7 @@ DefaultAction.prototype.pickCoordinate = function(event, preview) {
                 this.setState(DefaultAction.State.SettingReference);
             }
             else {
-                this.d2Model = event.getModelPosition();
+                this.d2Model = d2;
                 op = new RMoveReferencePointOperation(this.d1Model, this.d2Model);
                 op.setText(qsTr("Move Reference Point"));
                 this.di.applyOperation(op);
@@ -567,7 +586,7 @@ DefaultAction.prototype.selectEntity = function(entityId, add) {
 };
 
 /**
- * Called when the user selects a single entity.
+ * Called when the user double-clicks an entity.
  */
 DefaultAction.prototype.entityDoubleClicked = function(entityId, event) {
     if (isNull(this.document) || isNull(this.di)) {
@@ -587,6 +606,7 @@ DefaultAction.prototype.entityDoubleClicked = function(entityId, event) {
     }
     else if (isBlockReferenceEntity(entity)) {
         // in block text editing with double-click:
+        /*
         var blockId = entity.getReferencedBlockId();
         var block = this.document.queryBlock(blockId);
         if (!isNull(block)) {
@@ -613,19 +633,27 @@ DefaultAction.prototype.entityDoubleClicked = function(entityId, event) {
                 return;
             }
         }
+        */
 
-        if (RSettings.getBoolValue("GraphicsView/DoubleClickEditBlock", false)===true) {
+        if (RSettings.getBoolValue("GraphicsView/DoubleClickEditBlock", true)===true) {
             include("scripts/Block/Block.js");
+            EAction.handleUserMessage(qsTr("Editing block '%1'<br>Choose <i>Block > Return to Main Drawing</i> when done").arg(entity.getReferencedBlockName()));
             Block.editBlock(this.di, entity.getReferencedBlockName());
         }
     }
+    else if (isHatchEntity(entity)) {
+        if (RSettings.getBoolValue("GraphicsView/DoubleClickEditHatch", true)===true) {
+            include("scripts/Modify/EditHatch/EditHatch.js");
+            EditHatch.editHatch(entity);
+        }
+    }
     else if (isLineEntity(entity) || 
 		isArcEntity(entity) ||
 		isSplineEntity(entity) ||
 		isPolylineEntity(entity) ||
 		isEllipseEntity(entity)) {
 
-        if (RSettings.getBoolValue("GraphicsView/DoubleClickSelectContour", false)===true) {
+        if (RSettings.getBoolValue("GraphicsView/DoubleClickSelectContour", true)===true) {
             include("scripts/Select/SelectContour/SelectContour.js");
             var matchingEntities = SelectContour.getConnectedEntities(this.document, entityId, 0.001);
             var add = (event.modifiers().valueOf() === Qt.ShiftModifier.valueOf());
