@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2015 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2016 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -17,7 +17,8 @@
  * along with QCAD.
  */
 
-include("../View.js");
+include("scripts/View/View.js");
+include("scripts/Widgets/OptionsToolBar/OptionsToolBar.js");
 
 function OptionsToolBarFocus(guiAction) {
     View.call(this, guiAction);
@@ -28,19 +29,9 @@ OptionsToolBarFocus.prototype = new View();
 OptionsToolBarFocus.prototype.beginEvent = function() {
     View.prototype.beginEvent.call(this);
 
-    var appWin = EAction.getMainWindow();
-    var ot = EAction.getOptionsToolBar();
-    var widgets = getWidgets(ot);
-    for (var p in widgets) {
-        var w = widgets[p];
-        if (isOfType(w, QLineEdit) ||
-            isOfType(w, RMathLineEdit) ||
-            isOfType(w, QSpinBox) ||
-            isOfType(w, QComboBox)) {
-
-            w.setFocus(Qt.OtherFocusReason);
-            break;
-        }
+    var w = OptionsToolBar.getFirstInputWidget();
+    if (!isNull(w)) {
+        w.setFocus(Qt.OtherFocusReason);
     }
 
     this.terminate();
